@@ -19,11 +19,15 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
 from base_module import ranaModule
+import re
 import urllib
 import threading
 import time
 import geocoding
 import geonames
+import local_search
+
+DEFAULT_GOOGLE_API_KEY = "ABQIAAAAv84YYgTIjdezewgb8xl5_xTKlax5G-CAZlpGqFgXfh-jq3S0yRS6XLrXE9CkHPS6KDCig4gHvHK3lw"
 
 def getModule(m,d,i):
   return(onlineServices(m,d,i))
@@ -47,13 +51,212 @@ class onlineServices(ranaModule):
          * make sure there are no results returned after the button is pressed
          * remove the cancel button and the "working" overlay """
       self.stop()
+
+  def _disableOverlay(self):
+    """disable the "working" overlay + disable the timestamp"""
+    self.sendMessage('ml:notification:workInProgressOverlay:disable')
+    self.workStartTimestamp = None
+
+  def _onlineRouteLookup(self, query, type):
+    """this method online route lookup and is called by the worker thread"""
+    (start, destination, routeRequestSentTimestamp) = query
+    print "worker: routing from",start," to ",destination
+    self._setWorkStatusText("online routing in progress...")
+    # get the route
+    directions = self.googleDirections(start, destination)
+
+    if type == "LL":
+      # reverse geocode the start and destination coordinates (for the info menu)
+      (fromLat,fromLon) = start
+      (toLat,toLon) = destination
+      self._setWorkStatusText("geocoding start...")
+      startAddress = self.googleReverseGeocode(fromLat,fromLon)
+      self._setWorkStatusText("geocoding destination...")
+      destinationAddress = self.googleReverseGeocode(toLat,toLon)
+      # return the original start/dest coordinates
+      startLL = start
+      destinationLL = destination
+    else:
+      # signalize that the original start/dest coordinates are unknown
+      startAddress = start
+      destinationAddress = destination
+      startLL = None
+      destinationLL = None
+    self._setWorkStatusText("online routing done   ")
+    # return result to the thread to handle
+    return directions, startAddress, destinationAddress, startLL, destinationLL, routeRequestSentTimestamp
+
+  def geocode(self, address):
+    """synchronous geocoding"""
+    return geocoding.geocode(address)
+
+  def geocodeAsync(self, address, outputHandler, key):
+    """asynchronous geocoding"""
+    self._addWorkerThread(self._onlineGeocoding, [address], outputHandler, key)
+
+  def _onlineGeocoding(self, address):
+    self._setWorkStatusText("online geocoding in progress...")
+    result = self.geocode(address)
+    self._setWorkStatusText("online geocoding done   ")
+    return result
+
+  def localSearch(self, query, where=None, maxResults=8):
+    """Synchronous generic local search query
+    * if where is not specified, current position is used
+    * returns False if the search failed for some reason
+    """
+    # we use the Google Local Search backend at the moment
+
+    if where is None: # use current position coordinates
+      pos = self.get("pos", None)
+      if pos is None:
+        print("onlineServices: can't do local search - current location unknown")
+        return False
+      else:
+        lat, lon = pos
+        local = self.googleLocalQueryLL(query,lat,lon)
+        if local:
+          points = self._processGLSResponse(local)
+          return points
+        else:
+          return []
+    else: # use location description provided in where
+      queryString = "%s loc:%s" % (query, where)
+      local = self.googleLocalQuery(queryString, maxResults)
+      if local:
+        points = self._processGLSResponse(local)
+        return points
+      else:
+        return []
+
+  def localSearchLL(self, query, lat, lon):
+    """Synchronous generic local search query
+    * around a point specified by latitude and longitude"""
+
+    # we use the Google Local Search backend at the moment
+    local = self.googleLocalQueryLL(query,lat,lon)
+    if local:
+      points = self._processGLSResponse(local)
+      return points
+    else:
+      return []
+
+  def _processGLSResponse(self, response):
+    """load GLS results to LocalSearchPoint objects"""
+    results = response['responseData']['results']
+    points = []
+    for result in results:
+      point = local_search.GoogleLocalSearchPoint(result)
+      points.append(point)
+    return points
+
+
+  # ** Background processing **
+
+  def _enableOverlay(self):
+    """enable the "working" overlay + set timestamp"""
+    self.sendMessage('ml:notification:workInProgressOverlay:enable')
+    self.workStartTimestamp = time.time()
+
+  def _setWorkStatusText(self, text):
+    notification = self.m.get('notification', None)
+    if notification:
+      notification.setWorkInProgressOverlayText(text)
+
+  def _addWorkerThread(self, *args):
+    """start the worker thread and provide it the specified arguments"""
+    w = self.Worker(self,*args)
+    w.daemon = True
+    w.start()
+    self.workerThreads.append(w)
+
+  def _done(self, thread):
+    """a thread reporting it is done"""
+    # un-register the thread
+    self._unregisterWorkerThread(thread)
+    # if no other threads are working, disable the overlay
+    if not self.workerThreads:
+      self._disableOverlay()
+
+  def stop(self):
+    """called after pressing the cancel button"""
+    # disable the overlay
+    self._disableOverlay()
+    # tell all threads not to return results TODO: per thread cancelling
+    if self.workerThreads:
+      for thread in self.workerThreads:
+        thread.dontReturnResult()
+
+  def _unregisterWorkerThread(self, thread):
+    if thread in self.workerThreads:
+      self.workerThreads.remove(thread)
+
+  class Worker(threading.Thread):
+    """a worker thread for asynchronous online services access"""
+    def __init__(self,callback, call, args, outputHandler, key):
+      threading.Thread.__init__(self)
+      self.callback = callback # should be a onlineServices module instance
+      self.call = call
+      self.args = args
+      self.outputHandler = outputHandler
+      self.key = key # a key for the output handler
+      self.statusMessage = ""
+      self.returnResult = True
+    def run(self):
+      print("onlineServices: worker starting")
+      # enable the overlay
+      self.callback._enableOverlay()
+      # call the provided method asynchronously from modRana main thread
+      result = self.call(*self.args) # with the provided arguments
+      if self.returnResult: # check if our result is expected and should be returned to the oputpt handler
+        self.outputHandler(self.key, result)
+
+      # cleanup
+      print("onlineServices: worker finished")
+      self.callback._done(self)
+
+    def dontReturnResult(self):
+      self.returnResult = False
+
+
+  # ** OSM static map URL **
+
+  def getOSMStaticMapUrl(self, centerLat, centerLon, zl,
+                         w=350,h=350,
+                         defaultMarker = "ol-marker",
+                         markerList=[]
+                         ):
+    """construct & return OSM static map URL"""
+    prefix = "http://staticmap.openstreetmap.de/staticmap.php"
+    center = "?center=%f,%f" % (centerLat, centerLon)
+    zoom = "&zoom=%d" % zl
+    size = "&size=%dx%d" % (w,h)
+    if markerList:
+      markers= "&markers="
+      for marker in markerList:
+        if len(marker) == 2:
+          lat, lon = marker
+          markerName = defaultMarker
+        else:
+          lat, lon, markerName = marker
+        markers+= "%f,%f,%s|" % (lat, lon, markerName)
+    else:
+      markers = ""
+
+    url = "%s%s%s%s%s" % (prefix, center, zoom, size, markers)
+    # remove trailing | if present
+    if url[-1] == "|":
+      url = url[:-1]
+    return url
+
+  # ** Geonames **
       
   def elevFromGeonames(self, lat, lon):
     """get elevation in meters for the specified latitude and longitude from geonames"""
     url = 'http://ws.geonames.org/srtm3?lat=%f&lng=%f' % (lat,lon)
     try:
       query = urllib.urlopen(url)
-    except:
+    except Exception, e:
       "onlineServices: getting elevation from geonames returned an error"
       return 0
     return query.read()
@@ -66,10 +269,10 @@ class onlineServices(ranaModule):
       self._setWorkStatusText("online elevation lookup starting...")
       results = self.elevFromGeonamesBatch(latLonList)
       self._setWorkStatusText("online elevation lookup done   ")
-      return (results, tracklog) 
+      return results, tracklog
     except Exception, e:
-      print('onlineServices: exception suring elevation lookup:\n',e)
-      return (None,tracklog)
+      print('onlineServices: exception during elevation lookup:\n',e)
+      return None,tracklog
 
 
   def elevFromGeonamesBatch(self, latLonList):
@@ -92,48 +295,51 @@ class onlineServices(ranaModule):
         lats += "%f," % point[0]
         lons += "%f," % point[1]
 
-# TODO: maybe add switching ?
-#      url = 'http://ws.geonames.org/astergdem?lats=%s&lngs=%s' % (lats,lons)
+    # TODO: maybe add switching ?
+    #      url = 'http://ws.geonames.org/astergdem?lats=%s&lngs=%s' % (lats,lons)
       url = 'http://ws.geonames.org/srtm3?lats=%s&lngs=%s' % (lats,lons)
       try:
         query = urllib.urlopen(url)
-      except:
+      except Exception ,e:
         "onlineServices: getting elevation from geonames returned an error"
         results = "0"
         for i in range(1, len(tempList)):
-          results = results + " 0"
+          results += " 0"
       try:
         results = query.read().split('\r\n')
         query.close()
-      except:
+      except Exception, e:
         "onlineServices: elevation string from geonames has a wrong format"
         results = "0"
         for i in range(1, len(tempList)):
-          results = results + " 0"
+          results += " 0"
 
       index = 0
       for point in tempList: # add the results to the new list with elevation
         latLonElevList.append((point[0],point[1],int(results[index])))
-        index = index +1
+        index += 1
 
     return latLonElevList
 
+  # ** Google Maps **
+
   def getGmapsInstance(self):
     """get a google maps wrapper instance"""
-    key = self.get('googleAPIKey', None)
-    if key == None:
+    key = self.get('googleAPIKey', DEFAULT_GOOGLE_API_KEY)
+    if key is None:
       print "onlineServices: a google API key is needed for using the google maps services"
       return None
     # only import when actually needed
     import googlemaps
-    gmap = googlemaps.GoogleMaps(key)
-    return gmap
+    gMap = googlemaps.GoogleMaps(key)
+    return gMap
 
-  def googleLocalQuery(self, query):
+  def googleLocalQuery(self, query, maxResults=0):
     print "local search query: %s" % query
-    gmap = self.getGmapsInstance()
-    numresults = int(self.get('GLSResults', 8))
-    local = gmap.local_search(query, numresults)
+    gMap = self.getGmapsInstance()
+    if not maxResults:
+      maxResults = int(self.get('GLSResults', 8))
+    local = gMap.local_search(query, maxResults)
     return local
 
   def googleLocalQueryLL(self, term, lat, lon):
@@ -142,9 +348,13 @@ class onlineServices(ranaModule):
     return local
 
   def constructGoogleQueryLL(self, term, lat, lon):
-    """get a correctly formated GLS query"""
-    sufix = " loc:%f,%f" % (lat,lon)
-    query = term + sufix
+    """get a correctly formatted GLS query"""
+    query = "%s loc:%f,%f" % (term, lat,lon)
+    return query
+
+  def constructGoogleQuery(self, term, location):
+    """get a correctly formatted GLS query"""
+    query = "%s loc:%s" % (term, location)
     return query
 
   def googleDirectionsAsync(self, start, destination, outputHandler, key):
@@ -155,8 +365,8 @@ class onlineServices(ranaModule):
     self._addWorkerThread(self._onlineRouteLookup, [(start, destination, routeRequestSentTimestamp), "normal"], outputHandler, key)
     
   def googleDirectionsLLAsync(self, start, destination, outputHandler, key):
-    """a background running googledirections query
-    - Lat Lon pairsversion -> for geocoding the start/destination points (NOT first/last route points)
+    """a background running Google Directions query
+    - Lat Lon pairs version -> for geocoding the start/destination points (NOT first/last route points)
        outputHandler will be provided with the results + the specified key string"""
     routeRequestSentTimestamp = time.time() # used for measuring how long the route lookup took
     self._addWorkerThread(self._onlineRouteLookup, [(start, destination, routeRequestSentTimestamp), "LL"], outputHandler, key)
@@ -164,14 +374,14 @@ class onlineServices(ranaModule):
   def googleDirections(self ,start, destination):
     '''
     Get driving directions from Google.
-    start and directions can be either coordinates tupples or address strings
+    start and directions can be either coordinates tuples or address strings
     '''
 
     otherOptions=""
     if self.get('routingAvoidHighways', False): # optionally avoid highways
-      otherOptions = otherOptions + 'h'
+      otherOptions += 'h'
     if self.get('routingAvoidToll', False): # optionally avoid toll roads
-      otherOptions = otherOptions + 't'
+      otherOptions += 't'
 
     # respect travel mode
     mode = self.get('mode', None)
@@ -184,9 +394,9 @@ class onlineServices(ranaModule):
     else:
       type = ""
 
-     # combine type and aother parameters
+     # combine type and other parameters
     dir = {}
-    # the google language code is the seccond part of this whitespace delimited string
+    # the google language code is the second part of this whitespace delimited string
     googleLanguageCode = self.get('directionsLanguage', 'en en').split(" ")[1]
     dir['hl'] = googleLanguageCode
     directions = self.tryToGetDirections(start, destination, dir, type, otherOptions)
@@ -231,18 +441,8 @@ class onlineServices(ranaModule):
     address = gmap.latlng_to_address(lat,lon)
     return address
 
-  def _enableOverlay(self):
-    """enable the "working" overlay + set timestamp"""
-    self.sendMessage('ml:notification:workInProgressOverlay:enable')
-    self.workStartTimestamp = time.time()
-
-  def _disableOverlay(self):
-    """disable the "working" overlay + disable the timestamp"""
-    self.sendMessage('ml:notification:workInProgressOverlay:disable')
-    self.workStartTimestamp = None
-
   def googleLocalQueryLLAsync(self, term, lat, lon,outputHandler, key):
-    """asynchronous Google Local Search query for explicit lat, lon coordiantes"""
+    """asynchronous Google Local Search query for explicit lat, lon coordinates"""
     query = self.constructGoogleQueryLL(term, lat, lon)
     self.googleLocalQueryAsync(query, outputHandler, key)
     
@@ -250,6 +450,9 @@ class onlineServices(ranaModule):
     """asynchronous Google Local Search query for """
     print "onlineServices: GLS search"
     # TODO: we use a single thread for both routing and search for now, maybe have separate ones ?
+    # Local Search doesn't like the geo: prefix so we remove it
+    query = re.sub("loc:.*geo:", "loc:", query)
+
     self._addWorkerThread(self._localGoogleSearch, [query], outputHandler, key)
 
   def _localGoogleSearch(self, query):
@@ -260,46 +463,7 @@ class onlineServices(ranaModule):
     self._setWorkStatusText("online POI search done   ")
     return result
 
-  def _onlineRouteLookup(self, query, type):
-    """this method online route lookup and is called by the worker thread"""
-    (start, destination, routeRequestSentTimestamp) = query
-    print "worker: routing from",start," to ",destination
-    self._setWorkStatusText("online routing in progress...")
-    # get the route
-    directions = self.googleDirections(start, destination)
-
-    if type == "LL":
-      # reverse geocode the start and destination coordinates (for the info menu)
-      (fromLat,fromLon) = start
-      (toLat,toLon) = destination
-      self._setWorkStatusText("geocoding start...")
-      startAddress = self.googleReverseGeocode(fromLat,fromLon)
-      self._setWorkStatusText("geocoding destination...")
-      destinationAddress = self.googleReverseGeocode(toLat,toLon)
-      # return the original start/dest coordinates
-      startLL = start
-      destinationLL = destination
-    else:
-      # signalize that the original start/dest coordinates are unknown
-      startAddress = start
-      destinationAddress = destination
-      startLL = None
-      destinationLL = None
-    self._setWorkStatusText("online routing done   ")
-    # return result to the thread to handle
-    return (directions, startAddress, destinationAddress, startLL, destinationLL, routeRequestSentTimestamp)
-
-  def geocode(self, address):
-    return geocoding.geocode(address)
-
-  def geocodeAsync(self, address, outputHandler, key):
-    self._addWorkerThread(self._onlineGocoding, [address], outputHandler, key)
-
-  def _onlineGocoding(self, address):
-    self._setWorkStatusText("online geocoding in progress...")
-    result = self.geocode(address)
-    self._setWorkStatusText("online geocoding done   ")
-    return result
+  # ** Wikipedia search (through  Geonames) **
 
   def wikipediaSearch(self, query):
     return geonames.wikipediaSearch(query)
@@ -314,66 +478,6 @@ class onlineServices(ranaModule):
     return result
 
 
-  def _setWorkStatusText(self, text):
-    notification = self.m.get('notification', None)
-    if notification:
-      notification.setWorkInProgressOverlayText(text)
-
-  def _addWorkerThread(self, *args):
-    """start the worker thread and provide it the specified arguments"""
-    w = self.Worker(self,*args)
-    w.daemon = True
-    w.start()
-    self.workerThreads.append(w)
-
-  def _done(self, thread):
-    """a thread reporting it is done"""
-    # un-register the thread
-    self._unregisterWorkerThread(thread)
-    # if no other threads are working, disable the overlay
-    if not self.workerThreads:
-      self._disableOverlay()
-
-  def stop(self):
-    """called after pressing the cancel button"""
-    # disable the overlay
-    self._disableOverlay()
-    # tell all threads not to return results TODO: per thread cancelling
-    if self.workerThreads:
-      for thread in self.workerThreads:
-        thread.dontReturnResult()
-
-  def _unregisterWorkerThread(self, thread):
-    if thread in self.workerThreads:
-      self.workerThreads.remove(thread)
-
-  class Worker(threading.Thread):
-    """a worker thread for asynchronous online services access"""
-    def __init__(self,callback, call, args, outputHandler, key):
-      threading.Thread.__init__(self)
-      self.callback = callback # should be a onlineServicess module instance
-      self.call = call
-      self.args = args
-      self.outputHandler = outputHandler
-      self.key = key # a key for the output handler
-      self.statusMessage = ""
-      self.returnResult = True
-    def run(self):
-      print("onlineServices: worker starting")
-      # enable the overlay
-      self.callback._enableOverlay()
-      # call the provided method asynchronously from modRana main thread
-      result = self.call(*self.args) # with the provided arguments
-      if self.returnResult: # check if our result is expected and should be returned to the oputpt handler
-        self.outputHandler(self.key, result)
-            
-      # cleanup
-      print("onlineServices: worker finished")
-      self.callback._done(self)
-
-    def dontReturnResult(self):
-      self.returnResult = False
-
 if(__name__ == "__main__"):
   a = example({}, {})
   a.update()
