@@ -8,23 +8,26 @@ class Point:
     self.elevation = elevation # should be in meters
     self.message = message
 
-  def __str__(self):
-    if self.getElevation() == None:
+  def __unicode__(self):
+    if self.getElevation() is None:
       elev = "unknown"
     else:
       elev = "%f m" % self.getElevation()
 
-    return('%f,%f elevation: %s "%s"' % (self.lat, self.lon, elev, self.getMessage()))
+    return '%f,%f elevation: %s "%s"' % (self.lat, self.lon, elev, self.getMessage())
+
+  def __str__(self):
+    return unicode(self).encode('utf-8')
 
   def getLL(self):
-    return (self.lat,self.lon)
+    return self.lat,self.lon
 
   def setLL(self, lat, lon):
     self.lat = lat
     self.lon = lon
 
   def getLLE(self):
-    return (self.lat,self.lon, self.elevation)
+    return self.lat,self.lon, self.elevation
 
   def setLLE(self, lat, lon, elevation):
     self.lat = lat
@@ -32,10 +35,10 @@ class Point:
     self.elevation = elevation
 
   def getLLEM(self):
-    return (self.lat,self.lon, self.elevation, self.message)
+    return self.lat,self.lon, self.elevation, self.message
 
   def getElevation(self):
-    return (self.elevation)
+    return self.elevation
 
   def setElevation(self, elevation):
     self.elevation = elevation
